/*
    WMpop
    Copyright  1999-2002 Jrme SAUTRET (jerome@Sautret.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
/*#include <libintl.h> */
#include <locale.h> 
#include "wmpop.h"
#include "pop3.h"
#include "config_file.h"
#include "wmpop.xpm"

#ifdef DMALLOC
#include "dmalloc.h"
#endif


#define PARM_NORMAL 0
#define PARM_ERROR -1
#define PARM_NOFILE 1
#define PARM_SERVER 2
#define PARM_PASS 3

char file_name[BUF_SIZE];

/* widgets to use to write config file
*/
struct config_widgets
{
    GtkWidget *mbox, *pop3;
    GtkWidget *server, *name, *password;
    GtkWidget *mbox_check, *pop3_check;
    GtkWidget *default_pix, *nomail, *mail;
    GtkWidget *sound, *soundfile;
    GtkWidget *unread_mail, *command;
    GtkWidget *apop;
};


/********************************************************************/
/*                   functions declaration                          */
/********************************************************************/
void cancel_file_box(GtkWidget*, gpointer);
void ok_file_box(GtkWidget*, gpointer);
void destroy_box(GtkWidget *widget, gpointer button);
void main_win();
void wmpop_init(char*, 
		struct pop3_account*,
		struct config*);

/********************************************************************/
/*                   functions definitions                          */
/********************************************************************/

/* check the paramteres
*/
int check_parms(int argc, char *argv[])
{
    if (argc <= 1)  
	return PARM_ERROR;
    if (strcmp(argv[1],CONF_PARM) != 0 )
	return PARM_ERROR;
    if (argc >= 3)
    {
        if (strcmp(argv[2],"nofile") == 0 )
            return PARM_NOFILE;
        if (strcmp(argv[2],"pass") == 0 )
            return PARM_PASS;
        if (strcmp(argv[2],"server") == 0 )
            return PARM_SERVER;
    }

    return PARM_NORMAL;
}

/* set sensitive (or not, depend of boolean b) an array of n widgets
*/
void set_visible(GtkWidget *widget_array[], gboolean b)
{
    int i;

    for (i=0 ; widget_array[i] ; i++)
	gtk_widget_set_sensitive(GTK_WIDGET(widget_array[i]), b);
}

/* create a file selection box when a browse button is clicked,
   and set unactive this button
*/
void browse_file(GtkButton *button, char *title, GtkEntry *entry)
{
    GtkWidget *file_box;

    gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

    file_box = gtk_file_selection_new(title);
    
    gtk_signal_connect (GTK_OBJECT (file_box), "destroy",
			(GtkSignalFunc) destroy_box, button);
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					   (file_box)->cancel_button),
			       "clicked", (GtkSignalFunc) gtk_widget_destroy,
			       GTK_OBJECT(file_box));

    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION
                                    (file_box)->ok_button),
                        "clicked", (GtkSignalFunc) ok_file_box, entry);
    
    gtk_file_selection_set_filename (GTK_FILE_SELECTION(file_box), 
				     gtk_entry_get_text(GTK_ENTRY(entry)));
    gtk_widget_show(file_box);
}

/* fill pop3_account & config structures from config_widgets struct
*/
void create_config(struct config_widgets *widgets, 
		   struct pop3_account *account,
		   struct config *configuration)
{
    account->server=gtk_entry_get_text(GTK_ENTRY(widgets->server));
    account->apop =
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->apop));
    account->name  =gtk_entry_get_text(GTK_ENTRY(widgets->name));
    account->passwd=gtk_entry_get_text(GTK_ENTRY(widgets->password));
    account->port=DEFAULT_PORT;

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->mbox)))
    {
      configuration->proto = PROTO_MBOX;
    } 
    else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->pop3)))
    {
      configuration->proto = PROTO_POP3;
    } 
    configuration->pop3_check = 
      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgets->pop3_check))
      * 60;
    configuration->mbox_check = 
      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widgets->mbox_check));
    configuration->default_pix =
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->default_pix));
    configuration->pixmap[0]  = gtk_entry_get_text(GTK_ENTRY(widgets->nomail));
    configuration->pixmap[1]  = gtk_entry_get_text(GTK_ENTRY(widgets->mail));
    configuration->sound      =
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->sound));
    configuration->sound_file =
      gtk_entry_get_text(GTK_ENTRY(widgets->soundfile));
    configuration->command =
      gtk_entry_get_text(GTK_ENTRY(widgets->command));
    configuration->unread_mail =
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->unread_mail));
}

int write_file(char *file_name,
	       struct pop3_account *account,
	       struct config *configuration)
{
    FILE *file;

    file=fopen(file_name,"w");
    if (!file)
    {
        perror(file_name);
        return 0;
    }
    fprintf(file, _("# config file for WMpop\n"));
    fprintf(file, _("# automatically generated by conf_wmpop\n"));
    fprintf(file, _("# edit at your own risk\n\n"));
    switch (configuration->proto)
    {
    case PROTO_MBOX:
      fprintf(file, "proto \t\t\t%s\n", VALUE_PROTO_MBOX);
      break;
    case PROTO_POP3:
      fprintf(file, "proto \t\t\t%s\n", VALUE_PROTO_POP3);
      break;
    }
    fprintf(file, "mbox_check \t\t%ld\n", configuration->mbox_check);
    fprintf(file, "mailserver \t\t%s\n",    account->server);
    if(account->apop)
        fprintf(file, "apop\n");
    fprintf(file, "name \t\t\t%s\n",      account->name);
    fprintf(file, "password \t\t%s\n",      account->passwd);
    fprintf(file, "pop3_check \t\t%ld\n", configuration->pop3_check);
    if(!configuration->default_pix)
	fprintf(file, "custom_pix\n");
    fprintf(file, "pixmap1 \t\t%s\n",  configuration->pixmap[0]);
    fprintf(file, "pixmap2 \t\t%s\n",  configuration->pixmap[1]);
    if(configuration->sound)
	fprintf(file, "sound\n");
    fprintf(file, "soundfile \t\t%s\n",  configuration->sound_file);
    fprintf(file, "command \t\t%s\n",    configuration->command);
    if(configuration->unread_mail)
        fprintf(file, "unread_mail_only\n");
    fclose(file);
    chmod(file_name, S_IRUSR|S_IWUSR);
    return 1;
}


/****************** Callback Functions *********************/

void dialog_retry(GtkButton *button, gpointer p)
{
    gtk_exit(CONF_CHECK);
}

void dialog_cancel(GtkButton *button, gpointer p)
{
    gtk_exit(CONF_CLOSE);
}

void dialog_config(GtkButton *button, gpointer p)
{
    main_win();
    gtk_widget_destroy(gtk_widget_get_ancestor(GTK_WIDGET(button),
					       GTK_TYPE_WINDOW));
}

void quit_wmpop(GtkButton *button, gpointer p)
{
    gtk_exit(CONF_QUIT);
}

/* callback for the Apply button
*/
void save_file(GtkButton *button, gpointer pwidgets)
{
    struct config_widgets *widgets;
    struct pop3_account account;
    struct config configuration;

    widgets=(struct config_widgets*) pwidgets;
    create_config(widgets, &account, &configuration);

    write_file(file_name, &account, &configuration);

    kill(getppid(), SIGUSR1);
}

/* callback for the OK button
*/
void save_file_and_quit(GtkButton *button, gpointer pwidgets)
{
    save_file(button, pwidgets);
    gtk_exit(CONF_CHECK);
}

/* callback for the Close button
*/
void just_quit(GtkButton *button, gpointer pwidgets)
{
    gtk_exit(CONF_CLOSE);
}

/* about window
*/
void about(GtkButton *button, gpointer foobar)
{
    GtkWidget *win, *label, *box, *button_close;
    char s[BUF_SIZE];
    
    gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

    win=gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_title(GTK_WINDOW(win), _("About WMpop"));
    gtk_signal_connect (GTK_OBJECT (win), "destroy",
			(GtkSignalFunc) destroy_box, button);

    box=gtk_vbox_new(FALSE, 5);

    g_snprintf(s, BUF_SIZE, "WMpop %s", VERSION);
    label=gtk_label_new(s);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 5);

    g_snprintf(s, BUF_SIZE, _("by Jrme SAUTRET (Jerome@Sautret.org)"));
    label=gtk_label_new(s);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);

    g_snprintf(s, BUF_SIZE, "(c) 1999-2002");
    label=gtk_label_new(s);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);

    g_snprintf(s, BUF_SIZE, _("Licensed under GNU General Public Licence "
			      "(GPL). See COPYING file for more details."));
    label=gtk_label_new(s);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);

    g_snprintf(s, BUF_SIZE, _("Homepage: http://WMpop.Sautret.org"));
    label=gtk_label_new(s);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);

    button_close=gtk_button_new_with_label(_(" Close "));
    gtk_signal_connect_object (GTK_OBJECT (button_close), "clicked",
                               GTK_SIGNAL_FUNC(gtk_widget_destroy),
                               GTK_OBJECT (win));
    gtk_widget_show(button_close);
    gtk_box_pack_start(GTK_BOX(box), button_close, TRUE, TRUE, 5);

    gtk_widget_show(box);

    gtk_container_set_border_width(GTK_CONTAINER(win), 10);
    gtk_container_add(GTK_CONTAINER(win), box);

    gtk_widget_show(win);
}

/* remove the 's' of minutes if there's only one
   (totaly useless, of course)
*/
void changed_spin_check(GtkEditable *editable, gpointer label)
{
    if(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(editable)) > 1)
	gtk_label_set_text(GTK_LABEL(label), _("minutes"));
    else
	gtk_label_set_text(GTK_LABEL(label), _("minute "));
}

/* set sensitive the button that called the window
   when this window is destroyed.
*/
void destroy_box(GtkWidget *widget, gpointer button)
{
    gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);    
}

/* when OK is clicked on a file selection box, set the appropriate
   entry and destroy the box
*/
void ok_file_box(GtkWidget *widget, gpointer entry)
{
    GtkWidget *file_box;

    file_box=gtk_widget_get_ancestor(widget, GTK_TYPE_FILE_SELECTION);
    gtk_entry_set_text(GTK_ENTRY(entry),
		       gtk_file_selection_get_filename(
					     GTK_FILE_SELECTION(file_box)));
    gtk_entry_set_position(GTK_ENTRY(entry), 
			   strlen(gtk_entry_get_text(GTK_ENTRY(entry))));
    gtk_widget_destroy(file_box);
}

/* create a file selection box for choosing a pixmap no mail
*/
void browse_file_nomail(GtkButton *button, gpointer entry)
{
    browse_file(button,_("Choose a pixmap for no mail"), GTK_ENTRY(entry));
}

/* create a file selection box for choosing a pixmap mail
*/
void browse_file_mail(GtkButton *button, gpointer entry)
{
    browse_file(button,_("Choose a pixmap for mail received"),
		GTK_ENTRY(entry));
}

/* create a file selection box for choosing a sound file
*/
void browse_file_sound(GtkButton *button, gpointer entry)
{
    browse_file(button,_("Choose a sound file"), GTK_ENTRY(entry));
}

/* unsentitive widgets when default is clicked
*/
void  unset_sensitive(GtkToggleButton *togglebutton, gpointer w)
{
    if (gtk_toggle_button_get_active(togglebutton))
      set_visible(w, FALSE);
}

/* sentitive widgets when default is clicked
*/
void  set_sensitive(GtkToggleButton *togglebutton, gpointer w)
{
    if (gtk_toggle_button_get_active(togglebutton))
	set_visible(w, TRUE);
}

void  unset_and_set_sensitive(GtkToggleButton *togglebutton, gpointer w)
{
    if (gtk_toggle_button_get_active(togglebutton))
    {
	unset_sensitive(togglebutton, w);
	while(*( (GtkWidget**)w )++);
	set_sensitive(togglebutton, w);
    }
}

void  set_and_unset_sensitive(GtkToggleButton *togglebutton, gpointer w)
{
    if (gtk_toggle_button_get_active(togglebutton))
    {
	set_sensitive(togglebutton, w);
	while(*( (GtkWidget**)w )++);
	unset_sensitive(togglebutton, w);
    }
}

/*************** End Callback Functions ********************/


/* create the widget containing all the pop3 configuration stuffs
*/
GtkWidget *pop3_conf(struct pop3_account *account,
		     struct config *configuration,
		     struct config_widgets *widgets)
{
    GtkWidget *table_pop3, *label_server, *entry_server;
    GtkWidget *label_user, *entry_user, *label_password, *entry_password;
    GtkWidget *label_check, *spin_check, *label_min, *separator;
    GtkWidget *frame,*unread;
    GtkWidget *apop;
    GtkObject *adj_check;
    GtkTooltips *tooltips;

    tooltips=gtk_tooltips_new();

    label_server=gtk_label_new(_("Server Name:"));
    gtk_label_set_justify(GTK_LABEL(label_server), GTK_JUSTIFY_RIGHT);
    gtk_misc_set_alignment(GTK_MISC(label_server),1,0.5);
    gtk_widget_show(label_server);
    label_user=gtk_label_new(_("User Name:"));
    gtk_label_set_justify(GTK_LABEL(label_user), GTK_JUSTIFY_RIGHT);
    gtk_misc_set_alignment(GTK_MISC(label_user),1,0.5);
    gtk_widget_show(label_user);
    label_password=gtk_label_new(_("Password:"));
    gtk_misc_set_alignment(GTK_MISC(label_password),1,0.5);
    gtk_widget_show(label_password);

    apop=gtk_check_button_new_with_label(_("Use APOP"));
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(apop),
				account->apop); 
    gtk_widget_show(apop);
    widgets->apop=apop;

    gtk_tooltips_set_tip(tooltips, apop,
			 _("Use APOP protocol, which is a secure remplacement"
			   "for POP3."), NULL);


    label_check=gtk_label_new(_("Check mail every"));
    gtk_misc_set_alignment(GTK_MISC(label_check),1,0.5);
    gtk_widget_show(label_check);
    if(configuration->check/60 > 1)
	label_min=gtk_label_new(_("minutes"));
    else
	label_min=gtk_label_new(_("minute"));
    gtk_misc_set_alignment(GTK_MISC(label_min),0,0.5);
    gtk_label_set_justify(GTK_LABEL(label_min), GTK_JUSTIFY_LEFT);
    gtk_widget_show(label_min);

    unread=gtk_check_button_new_with_label(_("Check only unread mail"));
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(unread),
				configuration->unread_mail); 
    gtk_widget_show(unread);
    widgets->unread_mail=unread;

    entry_server=gtk_entry_new_with_max_length(127);
    gtk_widget_set_usize(entry_server, 150, 20);
    if (account->server)
	gtk_entry_set_text(GTK_ENTRY(entry_server), account->server);
    gtk_widget_show(entry_server);
    widgets->server=entry_server;

    gtk_tooltips_set_tip(tooltips, entry_server,
			 _("Enter the name of your POP3 server "
			   "(ex: pop.provider.com)"), NULL);

    entry_user=gtk_entry_new_with_max_length(127);
    gtk_widget_set_usize(entry_user, 150, 20);
    if (account->name)
	gtk_entry_set_text(GTK_ENTRY(entry_user), account->name);
    gtk_widget_show(entry_user);
    widgets->name=entry_user;

    gtk_tooltips_set_tip(tooltips, entry_user, 
			 _("Enter your POP3 account name"), NULL);

    entry_password=gtk_entry_new_with_max_length(127);
    gtk_widget_set_usize(entry_password, 150, 20);
    if (account->passwd)
	gtk_entry_set_text(GTK_ENTRY(entry_password), account->passwd);
    gtk_entry_set_visibility(GTK_ENTRY(entry_password), FALSE);
    gtk_widget_show(entry_password);
    widgets->password=entry_password;

    gtk_tooltips_set_tip(tooltips, entry_password,
			 _("Enter your POP3 account password"), NULL);

    adj_check=gtk_adjustment_new(5, 1, 1440, 1, 10, 0);
    spin_check=gtk_spin_button_new(GTK_ADJUSTMENT(adj_check), 0.5, 0);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_check),
			      configuration->pop3_check/60);
    gtk_widget_show(spin_check);
    widgets->pop3_check=spin_check;

    gtk_signal_connect(GTK_OBJECT (spin_check), "changed",
		       (GtkSignalFunc) changed_spin_check, label_min);

    separator=gtk_hseparator_new();
    gtk_widget_show(separator);

    table_pop3=gtk_table_new(10, 4, FALSE);
    gtk_table_attach(GTK_TABLE(table_pop3), label_server,
		     1, 2, 2, 3, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), entry_server,
		     2, 4, 2, 3, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), label_user,
		     1, 2, 3, 4, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), entry_user,
		     2, 4, 3, 4, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), label_password,
		     1, 2, 4, 5, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), entry_password,
		     2, 4, 4, 5, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), apop,
		     1, 2, 5, 6, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), separator,
		     0, 4, 6, 7, GTK_FILL,0,0,20);
    gtk_table_attach(GTK_TABLE(table_pop3), label_check,
		     1, 2, 7, 8, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), spin_check,
		     2, 3, 7, 8, 0,0,0,5);
    gtk_table_attach(GTK_TABLE(table_pop3), label_min,
		     3, 4, 7, 8, 0,0,0,25);
    gtk_table_attach(GTK_TABLE(table_pop3), unread,
		     1, 3, 8, 9, 0,0,0,5);
    
    gtk_container_set_border_width(GTK_CONTAINER(table_pop3), 10);
    gtk_widget_show(table_pop3);

    frame=gtk_frame_new(_("POP3 Configuration"));
    gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
    gtk_widget_show(frame);
    gtk_container_add(GTK_CONTAINER(frame), table_pop3);

    return frame;

}

/* Create the widget containing the personalisation stuffs:
   # pixmaps selector 
   # sound selector
*/
GtkWidget *perso_conf(struct config *configuration,
		      struct config_widgets *widgets)
{
    GtkWidget *label_sound, *radio_default, *entry_nomail;
    GtkWidget *entry_mail, *separator, *separator2;
    GtkWidget *button_nomail, *button_mail, *box_nomail;
    GtkWidget *box_mail, *radio_pix;
    GtkWidget *radio_nosound, *radio_sound, *button_sound;
    GtkWidget *box_sound, *entry_sound;
    GtkWidget *label_mail, *label_app, *label_nomail, *box_perso;
    GtkWidget *label_command, *entry_command;
    GtkWidget **t, *frame;
    GtkTooltips *tooltips;

    tooltips=gtk_tooltips_new();

    /* pixmaps */
    label_app=gtk_label_new(_("Appearance:"));
    gtk_label_set_pattern(GTK_LABEL(label_app), "___________");
    gtk_misc_set_alignment(GTK_MISC(label_app),0,0.5);
    gtk_widget_show(label_app);

    label_nomail=gtk_label_new(_("        No Mail "));
    gtk_widget_show(label_nomail);
    label_mail=gtk_label_new(  _("  Mail Received "));
    gtk_widget_show(label_mail);

    radio_default=gtk_radio_button_new_with_label (NULL, _("default"));
    gtk_widget_show(radio_default);
    widgets->default_pix=radio_default;

    gtk_tooltips_set_tip(tooltips, radio_default,
			 _("Use default pixmaps"), NULL);
    
    radio_pix=gtk_radio_button_new_with_label 
	(gtk_radio_button_group (GTK_RADIO_BUTTON (radio_default)),
	 _("use pixmaps:"));
    gtk_widget_show(radio_pix);

    gtk_tooltips_set_tip(tooltips, radio_pix, _("Use custom pixmaps"), NULL);

    entry_nomail=gtk_entry_new_with_max_length(127);
    gtk_widget_show(entry_nomail);
    widgets->nomail=entry_nomail;

    gtk_tooltips_set_tip(tooltips, entry_nomail,
			 _("Name of a 48x48 pixmap file (.xpm) "
			   "to display when no mail is received"), NULL);

    entry_mail=gtk_entry_new_with_max_length(127);
    gtk_widget_show(entry_mail);
    widgets->mail=entry_mail;

    gtk_tooltips_set_tip(tooltips, entry_mail, 
			 _("Name of a 48x48 pixmap "
			   "file (.xpm) to display when a mail is received"),
			 NULL);

    button_nomail=gtk_button_new_with_label(_("Browse..."));
    gtk_widget_show(button_nomail);

    gtk_signal_connect(GTK_OBJECT (button_nomail), "clicked",
		       (GtkSignalFunc) browse_file_nomail, entry_nomail);    

    button_mail=gtk_button_new_with_label(_("Browse..."));
    gtk_widget_show(button_mail);

    gtk_signal_connect(GTK_OBJECT (button_mail), "clicked",
		       (GtkSignalFunc) browse_file_mail, entry_mail);    


    if (configuration->pixmap[0])
    {
        gtk_entry_set_text(GTK_ENTRY(entry_nomail), configuration->pixmap[0]);
    }
    if (configuration->pixmap[1])
    {
        gtk_entry_set_text(GTK_ENTRY(entry_mail), configuration->pixmap[1]);
    }

    t=malloc(sizeof(GtkWidget*)*7);
    t[0]=label_mail;
    t[1]=entry_mail;
    t[2]=button_mail;
    t[3]=label_nomail;
    t[4]=entry_nomail;
    t[5]=button_nomail;
    t[6]=NULL;
    gtk_signal_connect(GTK_OBJECT (radio_default), "toggled",
		       (GtkSignalFunc) unset_sensitive, t);    
    gtk_signal_connect(GTK_OBJECT (radio_pix), "toggled",
		       (GtkSignalFunc) set_sensitive, t);    

    if (!configuration->default_pix)
    {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_pix), TRUE);
        set_sensitive(GTK_TOGGLE_BUTTON(radio_pix),t);
    }
    else
    {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_default), TRUE);
        unset_sensitive(GTK_TOGGLE_BUTTON(radio_default),t);
    }

    box_nomail=gtk_hbox_new(FALSE,0);
    gtk_box_pack_start(GTK_BOX(box_nomail), label_nomail,  TRUE,  TRUE,  0);
    gtk_box_pack_start(GTK_BOX(box_nomail), entry_nomail,  FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_nomail), button_nomail, FALSE, FALSE, 3);
    gtk_widget_show(box_nomail);

    box_mail=gtk_hbox_new(FALSE,0);
    gtk_box_pack_start(GTK_BOX(box_mail), label_mail,  TRUE,  TRUE,  0);
    gtk_box_pack_start(GTK_BOX(box_mail), entry_mail,  FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_mail), button_mail, FALSE, FALSE, 3);
    gtk_widget_show(box_mail);

    separator=gtk_hseparator_new();
    gtk_widget_show(separator);

    /* sound */
    label_sound=gtk_label_new(_("Sound configuration:"));
    gtk_label_set_pattern(GTK_LABEL(label_sound), "____________________");
    gtk_misc_set_alignment(GTK_MISC(label_sound),0,0.5);
    gtk_widget_show(label_sound);

    radio_nosound=gtk_radio_button_new_with_label(NULL, _("No Sound"));
    gtk_widget_show(radio_nosound);
    gtk_tooltips_set_tip(tooltips, radio_nosound, _("Don't use sound"), NULL);

    radio_sound=  gtk_radio_button_new_with_label
	(gtk_radio_button_group(GTK_RADIO_BUTTON(radio_nosound)),
	 _("Play Sound File:"));
    gtk_widget_show(radio_sound);
    widgets->sound=radio_sound;
    gtk_tooltips_set_tip(tooltips, radio_sound,
			 _("Play a sound when a mail is received"), NULL);

    entry_sound=gtk_entry_new_with_max_length(127);
    if (configuration->sound_file)
    {
        gtk_entry_set_text(GTK_ENTRY(entry_sound), configuration->sound_file);
    }
    gtk_widget_show(entry_sound);
    widgets->soundfile=entry_sound;

    gtk_tooltips_set_tip(tooltips, entry_sound,
			 _("Name of a sun audio file (.au) "
			   "to play when a mail is received"), NULL);


    button_sound=gtk_button_new_with_label(_("Browse..."));
    gtk_widget_show(button_sound);

    gtk_signal_connect(GTK_OBJECT (button_sound), "clicked",
		       (GtkSignalFunc) browse_file_sound, entry_sound);    


    t=malloc(sizeof(GtkWidget*)*3);
    t[0]=entry_sound;
    t[1]=button_sound;
    t[2]=NULL;
    gtk_signal_connect(GTK_OBJECT (radio_nosound), "toggled",
		       (GtkSignalFunc) unset_sensitive, t);    
    gtk_signal_connect(GTK_OBJECT (radio_sound), "toggled",
		       (GtkSignalFunc) set_sensitive, t);    

    if (configuration->sound)
    {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_sound), TRUE);
        set_sensitive(GTK_TOGGLE_BUTTON(radio_sound),t);
    }
    else
    {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_nosound), TRUE);
        unset_sensitive(GTK_TOGGLE_BUTTON(radio_nosound),t);
    }    

    box_sound=gtk_hbox_new(FALSE,0);
    gtk_box_pack_start(GTK_BOX(box_sound), radio_sound, TRUE,  TRUE,  0);
    gtk_box_pack_start(GTK_BOX(box_sound), entry_sound,  FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_sound), button_sound, FALSE, FALSE, 3);
    gtk_widget_show(box_sound);

    separator2=gtk_hseparator_new();
    gtk_widget_show(separator2);

    /* external command */
    label_command = gtk_label_new(_("External command:"));
    gtk_label_set_pattern(GTK_LABEL(label_command), "________________");
    gtk_misc_set_alignment(GTK_MISC(label_command),0,0.5);
    gtk_widget_show(label_command);

    entry_command = gtk_entry_new_with_max_length(127);
    if (configuration->command)
    {
        gtk_entry_set_text(GTK_ENTRY(entry_command), configuration->command);
    }
    widgets->command=entry_command;
    gtk_widget_show(entry_command);

    gtk_tooltips_set_tip(tooltips, entry_command,
			 _("External command launched by the middle button")
			 , NULL);

    /* personalization box */
    box_perso=gtk_vbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(box_perso), label_app,         FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), radio_default,     FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), radio_pix,         FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), box_nomail,        FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), box_mail,          FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), separator,         TRUE, TRUE, 10);
    gtk_box_pack_start(GTK_BOX(box_perso), label_sound,       FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), radio_nosound,     FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), box_sound,         FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), separator2,        TRUE, TRUE, 10);
    gtk_box_pack_start(GTK_BOX(box_perso), label_command,     FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(box_perso), entry_command,     FALSE, FALSE, 0);
    
    gtk_container_set_border_width(GTK_CONTAINER(box_perso), 10);
    gtk_widget_show(box_perso);

    frame=gtk_frame_new(_("Personalization Parameters"));
    gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
    gtk_widget_show(frame);
    gtk_container_add(GTK_CONTAINER(frame), box_perso);

    return frame;
}

/* create the widget containing all the mail configuration stuffs
*/
GtkWidget *mail_conf( struct pop3_account *account,
		      struct config *configuration,
		      struct config_widgets *widgets)
{
  GtkWidget *radio_mbox, *radio_pop3, *vbox, *frame, *pop3, *spin_check;
  GtkWidget **t, *label, *hbox;
  GtkObject *adj_check;
  GtkTooltips *tooltips;

  pop3 = pop3_conf(account, configuration, widgets);

  radio_mbox = gtk_radio_button_new_with_label (NULL, _("Local mail:"));
  radio_pop3 = gtk_radio_button_new_with_label_from_widget(
				    GTK_RADIO_BUTTON(radio_mbox), _("POP3:"));
  
  /*gtk_label_set_pattern(GTK_LABEL(label_sound), "____________________");*/

  label = gtk_label_new(_("Mail checking interval (in seconds) :"));
  gtk_misc_set_alignment(GTK_MISC(label),1,0.5);
  gtk_widget_show(label);
  adj_check = gtk_adjustment_new(5, 1, 1440, 1, 10, 0);
  spin_check = gtk_spin_button_new(GTK_ADJUSTMENT(adj_check), 0.5, 0);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_check), 
			    configuration->mbox_check);

  hbox = gtk_hbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(hbox), label,       TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(hbox), spin_check,  TRUE,  TRUE,  0);

  t=malloc(sizeof(GtkWidget*) * 4);
  t[0]=pop3;
  t[1]=NULL;
  t[2]=hbox;
  t[3]=NULL;
  gtk_signal_connect(GTK_OBJECT (radio_mbox), "toggled",
		     (GtkSignalFunc) unset_and_set_sensitive, t);    
  gtk_signal_connect(GTK_OBJECT (radio_pop3), "toggled",
		     (GtkSignalFunc) set_and_unset_sensitive, t);
  
  switch (configuration->proto)
  {
  case PROTO_MBOX:
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_mbox), TRUE);
    unset_and_set_sensitive(GTK_TOGGLE_BUTTON(radio_mbox),t);
    break;
  case PROTO_POP3:
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_pop3), TRUE);
    set_and_unset_sensitive(GTK_TOGGLE_BUTTON(radio_pop3),t);
    break;
  }

  widgets->mbox = radio_mbox;
  widgets->pop3 = radio_pop3;
  widgets->mbox_check = spin_check;

  gtk_widget_show(radio_pop3);
  gtk_widget_show(radio_mbox);
  gtk_widget_show(label);
  gtk_widget_show(spin_check);
  gtk_widget_show(hbox);

  tooltips=gtk_tooltips_new();
  gtk_tooltips_set_tip(tooltips, radio_mbox,
			 _("Check mailbox (mbox format), defined in $MAIL, "
			   "on local machine."), NULL);
  gtk_tooltips_set_tip(tooltips, radio_pop3,
			 _("Check remote mailbox on a POP3 server"), NULL);
  gtk_tooltips_set_tip(tooltips, spin_check,
			 _("Number of seconds between each check of the "
			   "local mailbox"), NULL);

  vbox = gtk_vbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox), radio_mbox,  TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox,        TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(vbox), radio_pop3,  TRUE,  TRUE,  0);
  gtk_box_pack_start(GTK_BOX(vbox), pop3,        TRUE,  TRUE,  0);
  gtk_widget_show(vbox);

  frame=gtk_frame_new(_("Mail Checking"));
  gtk_container_set_border_width(GTK_CONTAINER(frame), 10);
  gtk_widget_show(frame);
  gtk_container_add(GTK_CONTAINER(frame), vbox);

  return frame;

}

/* create the widget contained in the main window
*/
GtkWidget *main_widget(struct pop3_account *account,
		       struct config *configuration)
{
    GtkWidget *winhbox, *notebook, *wid_mail, *wid_perso;
    GtkWidget *label_perso, *label_mail, *winvbox, *button_quit;
    GtkWidget *box_button, *button_ok, *button_apply, *button_cancel;
    GtkWidget *button_about;
    GtkTooltips *tooltips;
    static struct config_widgets widgets;

    tooltips=gtk_tooltips_new();

    label_mail=gtk_label_new(_("Mail Checking"));
    gtk_widget_show(label_mail);

    wid_mail=mail_conf(account, configuration, &widgets);
    wid_perso=perso_conf(configuration, &widgets);

    label_perso=gtk_label_new(_("Personalization"));
    gtk_widget_show(label_perso);

    notebook=gtk_notebook_new();
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), wid_mail,  label_mail);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), wid_perso, label_perso);
    gtk_widget_show(notebook);

    button_ok=gtk_button_new_with_label(_(" OK "));
    gtk_signal_connect(GTK_OBJECT (button_ok), "clicked",
		       (GtkSignalFunc) save_file_and_quit, &widgets);
    gtk_widget_show(button_ok);
    gtk_tooltips_set_tip(tooltips, button_ok,
			 _("Save the configuration file and "
			   "close this window"), NULL);

    button_apply=gtk_button_new_with_label(_(" Apply "));
    gtk_signal_connect(GTK_OBJECT (button_apply), "clicked",
		       (GtkSignalFunc) save_file, &widgets);
    gtk_widget_show(button_apply);
    gtk_tooltips_set_tip(tooltips, button_apply, 
			 _("Save the configuration file"), NULL);

    button_cancel=gtk_button_new_with_label(_(" Close "));
    gtk_signal_connect(GTK_OBJECT (button_cancel), "clicked",
		       (GtkSignalFunc) just_quit, NULL);
    gtk_widget_show(button_cancel);
    gtk_tooltips_set_tip(tooltips, button_cancel,
			 _("Close this window without saving anything"), NULL);

    button_about=gtk_button_new_with_label(_(" About "));
    gtk_signal_connect(GTK_OBJECT (button_about), "clicked",
		       (GtkSignalFunc) about, NULL);
    gtk_widget_show(button_about);

    button_quit=gtk_button_new_with_label(_(" Quit "));
    gtk_signal_connect(GTK_OBJECT (button_quit), "clicked",
		       (GtkSignalFunc) quit_wmpop, NULL);
    gtk_widget_show(button_quit);
    gtk_tooltips_set_tip(tooltips, button_quit,
			 _("Quit WMpop without saving anything"), NULL);

    box_button=gtk_hbox_new(TRUE, 10);
    gtk_box_pack_start(GTK_BOX(box_button), button_ok,     TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(box_button), button_apply,  TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(box_button), button_cancel, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(box_button), button_about,  TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(box_button), button_quit,  TRUE, TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(box_button), 2);
    gtk_widget_show(box_button);

    winvbox=gtk_vbox_new(FALSE, 3);
    gtk_box_pack_start(GTK_BOX(winvbox), notebook, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(winvbox), box_button, TRUE, TRUE, 0);
    gtk_widget_show(winvbox);
    

    winhbox=gtk_hbox_new(FALSE, 2);
    gtk_box_pack_end(GTK_BOX(winhbox), winvbox, TRUE, TRUE, 0);
    gtk_widget_show(winhbox);

    return winhbox;
}

void main_win()
{
    GtkWidget *win, *wid_win, *pixmap_wmpop;
    GdkPixmap *pixmap;
    struct pop3_account account;
    struct config configuration;
    
    DEBUG(printf("main win\n"));
    wmpop_init(file_name, &account, &configuration);

    win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(win), _("WMpop Configuration"));
    gtk_signal_connect(GTK_OBJECT (win), "delete_event",
		       (GtkSignalFunc) gtk_exit, NULL);

    wid_win=main_widget(&account, &configuration);

    gtk_container_add(GTK_CONTAINER(win), wid_win);
    gtk_container_set_border_width(GTK_CONTAINER(win), 5);

    gtk_widget_show(win);

    pixmap=gdk_pixmap_create_from_xpm_d(win->window, NULL, NULL, wmpop_xpm);
    pixmap_wmpop=gtk_pixmap_new(pixmap, NULL);
    gtk_widget_show(pixmap_wmpop);
    
    gtk_box_pack_start(GTK_BOX(wid_win), pixmap_wmpop,FALSE, FALSE, 0);

}

void create_dialog(char *s)
{
  GtkWidget *dialog, *label;
    GtkWidget *button_retry, *button_cancel, *button_config, *button_quit;

    dialog=gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dialog), "WMpop");
    gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);

    label=gtk_label_new(s);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
                        label, TRUE, TRUE, 0);
    gtk_widget_show(label);

    button_retry=gtk_button_new_with_label(_("Retry"));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
                        button_retry, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT (button_retry), "clicked",
		       (GtkSignalFunc) dialog_retry, NULL);
    gtk_widget_show (button_retry);

    button_cancel=gtk_button_new_with_label(_("Cancel"));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
                        button_cancel, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT (button_cancel), "clicked",
		       (GtkSignalFunc) dialog_cancel, NULL);
    gtk_widget_show (button_cancel);

    button_config=gtk_button_new_with_label(_("Edit Configuration"));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
                        button_config, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT (button_config), "clicked",
		       (GtkSignalFunc) dialog_config, NULL);
    gtk_widget_show (button_config);

    button_quit=gtk_button_new_with_label(_("Quit"));
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
                        button_quit, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT (button_quit), "clicked",
		       (GtkSignalFunc) quit_wmpop, NULL);
    gtk_widget_show (button_quit);

    gtk_widget_show (dialog);    
}

/* read config file & initialize values in account & configuration
*/
void wmpop_init(char *file_name, 
		struct pop3_account *account,
		struct config *configuration)
{
    int i;

    configuration->sound_file=NULL;
    memset(account, 0, sizeof(struct pop3_account));
    init_default_config(configuration, account);
    read_config_file(file_name, configuration, account);
 

    for (i=0;i<2;i++)
	if (configuration->pixmap[i] && *(configuration->pixmap[i])=='\0')
        {
            free(configuration->pixmap[i]);
            configuration->pixmap[i]=NULL;
        }
    if (configuration->sound_file && *(configuration->sound_file)=='\0')
    {
        free(configuration->sound_file);
        configuration->sound_file=NULL;
    }

}

/* create the gtk init & the main window
*/
int main(int argc, char *argv[])
{
    char s[BUF_SIZE];

    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);

    gtk_set_locale();
    gtk_init(&argc, &argv);

    /*config_file_name(file_name, argc, argv);*/
    
    switch(check_parms(argc, argv))
    {
	case PARM_NORMAL:
	    strcpy(file_name, argv[2]);
	    main_win();
	    break;
	case PARM_NOFILE:
	    strcpy(file_name, argv[3]);
	    g_snprintf(s, BUF_SIZE, 
		       _("No configuration file %s was found. "
			 "Peharps this is the first time your run "
			 "this program. You must configure WMpop "
			 "before using it.\nClick on the "
			 "'Edit Configuration' button below to configure."),
		       file_name);
	    create_dialog(s);
	    break;
	case PARM_PASS:
	    strcpy(file_name, argv[3]);
	    g_snprintf(s, BUF_SIZE, 
		       _("Can't log, pop server replied:\n%s\n"
			 "Peharps your user name and password are incorrect "
			 "or the server is already locked by another "
			 "connection."), 
		       argv[4]);
	    create_dialog(s);
	    break;
	case PARM_SERVER:
	    strcpy(file_name, argv[3]);
	    create_dialog(_("Can't find POP3 server. Check your server name"));
	    break;
	case PARM_ERROR:
	    fprintf(stderr, 
		    _("Sorry, this program must not be run directly, "
		      "run wmpop insteed.\n"));
	    return -1;
    }

    gtk_main();

    return 0;
}
