/*
  WMpop
  Copyright  1999-2002 Jrme SAUTRET (jerome@Sautret.org)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include "wmpop.h"
#include "pop3.h"
#include "x.h"
#include "config_file.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif


/*extern struct pop3_account account;
  extern struct config configuration;
*/

/* return the config file name in file_name parameter.
   The command line switch is first searched for the file name,
   if not found, defaut one is used.
 */
void config_file_name(char *file_name, int argc, char *argv[])
{
    int i;

    for (i=0 ; i < argc-1 ; i++)
    {
	if (strcmp(argv[i], SWITCH_CONFIG_FILE_SHORT) == 0)
	{
	    strncpy(file_name, argv[i+1], BUF_SIZE);
	    file_name[BUF_SIZE-1] = '\0';
	    return;
	}
    } 
    sprintf(file_name,"%s/%s",getenv("HOME"), CONFIG_FILE);
}

void reinit_string(char* s)
{
    if (s && *s)
    {
	free(s);
    }
}

void init_default_config(struct config *conf, struct pop3_account *account)
{
    int i;

    DEBUG(printf("begin init_default_config\n"))
    reinit_string(account->server);
    account->server="";
    account->apop=0;
    reinit_string(account->name);
    account->name="";
    reinit_string(account->passwd);
    account->passwd="";
    account->port=DEFAULT_PORT;

    for(i=0;i<2;i++)
    {
	if (conf->pixmap[i])
	{
	    /*free(conf->pixmap[i]);*/
	    conf->pixmap[i]=NULL;
	}
    }
    conf->proto = PROTO_MBOX;
    conf->check = DEFAULT_CHECK*60;
    conf->pop3_check = DEFAULT_CHECK * 60;
    conf->mbox_check = DEFAULT_CHECK;
    conf->default_pix=1;
    conf->sound=0;
    conf->unread_mail=0;
    conf->display_emails=1;
    conf->command="Eterm -x -w1 -t mutt -e mutt";

    if (conf->sound_file)
    {
	free(conf->sound_file);
	/* conf->sound_file=NULL; */
    }
    conf->sound_file=
	(char *) malloc( sizeof(char) * 
			 (strlen(DATADIR)+strlen(DEFAULT_SOUND_FILE)
			  + 2));
    strcpy(conf->sound_file, DATADIR);
    strcat(conf->sound_file, "/");
    strcat(conf->sound_file, DEFAULT_SOUND_FILE);
}


/*
Convert the string s read from config file to a time interval as a integer 
*/
int to_time(char *s)
{
    int t;

    t = atol(s);
    if (t <= 0)
    {
	fprintf(stderr, _("WMpop warning: invalid value %s for time interval, "
			  "default value used.\n"), s); 
	return -1;
    }
    return t;
}

void read_key_value(char *p, char* key, char *value_ptr)
{
    char *value;

    /* skip heading whitespaces */
    while(*p && ( *p == ' ' || *p == '\t' || *p == '\n' ) )
    {
	p++;
    }
    
    /* read key */
    while(*p && ( *p != ' ' && *p != '\t' && *p != '\n' ) )
    {
	*key++ = *p++;
    }
    *key   = '\0';

    /* skip whitespaces between key and value */
    while(*p && ( *p == ' ' || *p == '\t' ) )
    {
	p++;
    }

    /* read value */
    value = value_ptr; 
    while(*p && *p != '\n')
    {
	*value++ = *p++;
    }

    /* remove trailing whitespaces in value */
    value--;
    while(value >= value_ptr && (*value == ' ' || *value == '\t' ) )
    {
	value--;
    }
    *++value='\0';
}

int read_config_file(char *file_name,
		     struct config *configuration,
		     struct pop3_account *account)
{
    FILE *file;
    static char key[BUF_SIZE], value[BUF_SIZE], s[BUF_SIZE];

    file=fopen(file_name,"r");
    if (!file)
    {
	perror(file_name);
	return 0;
    }

    while (fgets(s, BUF_SIZE, file))
    {
	*key='\0';
	*value='\0';
	/*sscanf(s,"%s %1024s\n",key, value);*/
	/*DEBUG(printf("chaine %s\n", s));*/
	read_key_value(s, key, value);
	/*DEBUG(printf("key %s, value %s\n", key, value));*/
	if (*key!=COMMENT && *key!='\0')
	{
	    if (strcmp(key, KEY_PROTO) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		if (strcmp(value, VALUE_PROTO_POP3) == 0)
		{
		    DEBUG(printf("POP3 used\n"));
		    configuration->proto = PROTO_POP3;
		}
		else if (strcmp(value, VALUE_PROTO_MBOX) == 0)
		{
		    DEBUG(printf("MBOX used\n"));
		    configuration->proto = PROTO_MBOX;
		}
		else
		{
		    fprintf(stderr,
			    _("WMpop error: unknown protocol \"%s\".\n"),
			    value);
		    exit(3);
		}
	    }
	    else if (strcmp(key,KEY_SERVER) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		account->server=malloc(strlen(value)+1);
		strcpy(account->server, value);
	    }
	    else if (strcmp(key,KEY_NAME) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		account->name=malloc(strlen(value)+1);
		strcpy(account->name, value);
	    }
	    else if (strcmp(key,KEY_PASSWD) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		account->passwd=malloc(strlen(value)+1);
		strcpy(account->passwd, value);
	    }
	    else if (strcmp(key,KEY_PIX1) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		if (value)
		{
		    configuration->pixmap[0]=malloc(strlen(value)+1);
		    strcpy(configuration->pixmap[0], value);
		}
	    }
	    else if (strcmp(key,KEY_PIX2) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		if (value)
		{
		    configuration->pixmap[1]=malloc(strlen(value)+1);
		    strcpy(configuration->pixmap[1], value);
		}
	    }
	    else if (strcmp(key,KEY_COMMAND) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		if (value)
		{
		    configuration->command = malloc(strlen(value)+1);
		    strcpy(configuration->command, value);
		}
	    }
	    else if (strcmp(key,KEY_CHECK_POP3) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		if (value)
		{
		    configuration->pop3_check = to_time(value);
		    if (configuration->pop3_check == -1)
			configuration->pop3_check = DEFAULT_CHECK * 60;
		}
	    }
	    else if (strcmp(key,KEY_CHECK_MBOX) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		if (value)
		{
		    configuration->mbox_check = to_time(value);
		    if (configuration->mbox_check == -1)
			configuration->mbox_check = DEFAULT_CHECK;
		}
	    }
	    else if (strcmp(key,KEY_CUSTOM) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		configuration->default_pix=0;
	    }
	    else if (strcmp(key,KEY_SOUND) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		configuration->sound=1;
	    }
	    else if (strcmp(key,KEY_UNREAD_MAIL) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		configuration->unread_mail=1;
	    }
	    else if (strcmp(key,KEY_APOP) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		account->apop = 1;
	    }
	    else if (strcmp(key,KEY_SOUNDFILE) == 0)
	    {
		DEBUG(printf("k:%s, \tv:%s\n",key,value));
		if (value)
		{
		    if (configuration->sound_file)
			free(configuration->sound_file);
		    configuration->sound_file=malloc(strlen(value)+1);
		    strcpy(configuration->sound_file, value);
		}
	    }
	    else
	    {
		fprintf(stderr, _("WMpop warning: unknown keyword %s "
				  "in file %s, line ignored.\n"), 
			key, file_name);
	    }
	
	}
	switch (configuration->proto)
	{
	case PROTO_MBOX:
	    configuration->check = configuration->mbox_check;
	    break;
	case PROTO_POP3:
	    configuration->check = configuration->pop3_check;
	    break;
	default:
	    fprintf(stderr, _("WMpop internal error: unknown proto %d.\n"), 
		    configuration->proto);
	    exit(5);
	}
    }
    fclose(file);
    return 1;
}


    
