/*
    WMpop
    Copyright  1999-2002 Jrme SAUTRET (jerome@Sautret.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
/*#include <libintl.h> */
/*#include <resolv.h>*/
#include "wmpop.h"
#include "md5.h"
#include "pop3.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

extern int verbose;

/* declarations */
int pop3_connect(const struct pop3_account*);
int pop3_disconnect(int);
int pop3_recv(int, char*);
int pop3_send(int, char*);

/* return a socket connected to the mail server 
   or a code <0 if an error occured 
*/
int pop3_connect(const struct pop3_account *account)
{
    int sock;
    struct hostent *host;
    struct sockaddr_in addr;
        /*struct in_addr *addr_temp;*/

    sock=socket(AF_INET, SOCK_STREAM, 0);
    if (sock == -1)
    {
        perror("pop3_connect, socket");
        return -4;
    }

    host=gethostbyname(account->server);
    if (!host)
    {
        pop3_disconnect(sock);
        herror(account->server);
        if ( (h_errno == HOST_NOT_FOUND) || (h_errno == NO_ADDRESS))
            return -3;
        return -4;
    }
    
    addr.sin_family=host->h_addrtype;
    addr.sin_port=htons(account->port);
        /*   addr_temp=(struct in_addr *) host->h_addr_list[0];
             addr.sin_addr.s_addr=addr_temp->s_addr;*/
        /*    addr.sin_addr= *((struct in_addr *) host->h_addr_list[0]);*/
    addr.sin_addr.s_addr= *((unsigned long *) host->h_addr_list[0]);
    
    if (connect (sock, (struct sockaddr*) &addr, sizeof(addr)) == -1)
    {
        pop3_disconnect(sock);
        perror("pop3_connect, connect");
        return -1;
    }
    
    return sock;
}

int pop3_disconnect(int sock)
{
    if (shutdown(sock,2) == -1)
    {
        close(sock);
        perror("pop3_disconnect, shutdown");
        return 0;
    }
    close(sock);
    return 1;
}

/* send a string via the socket
*/
int pop3_recv(int sock, char *buf)
{
    int n;

    n=read(sock,buf,BUF_SIZE-1);
    if (n == -1)
    {
        perror ("pop3_recv, read");
        return 0;
    }
    buf[n]='\0';
    DEBUG(printf ("-> %s\n",buf))

        return 1;
}

/* receive a string from the socket
*/
int pop3_send(int sock, char *buf)
{
    if (write(sock,buf,strlen(buf)) == -1)
    {
        perror ("pop3_send, write");
        return 0;
    }
    DEBUG(printf ("<- %s\n",buf))
        return 1;
}

/* check mail on a pop3 mailserver and return 0 if OK,
   return -1 if an error occured.
   return -2 if logging is impossible (name or password is incorrect or account
   is already locked by another connexion.
   Set the fields of emails to read values.
*/
int pop3_check(struct emails_t *emails, struct pop3_account *account)
{
    int sock;
    int nb,size,last;
    int i;
    char buffer[BUF_SIZE];
    unsigned char md5digest[16];
    char hexdigits[] = "0123456789abcdef";
    char md5cdigest[33];
    char *tstamp_beg,*tstamp_end;
    MD5_CTX md5context;

    sock=pop3_connect(account);
    if (sock < 0)
	return sock;
    pop3_recv(sock,buffer);

    /* Search for APOP timestamp */
    tstamp_beg = strchr(buffer, '<');
    if (tstamp_beg)
        tstamp_end = strchr(buffer, '>');

    /* APOP or classical authentication ? */
    if (account->apop)
    {
        if (!tstamp_beg || (tstamp_end <= tstamp_beg))
        {
          fprintf(stderr,_("APOP unavailable on server %s\n"),account->server);
          return -2;
        }
        MD5Init(&md5context);
        MD5Update(&md5context,tstamp_beg,(tstamp_end - tstamp_beg + 1));
        MD5Update(&md5context,account->passwd,strlen(account->passwd));
        MD5Final(md5digest,&md5context);
        for (i = 0; i < 16; i++)
        {
            md5cdigest[i * 2] = hexdigits[md5digest[i] >> 4];
            md5cdigest[i * 2 + 1] = hexdigits[md5digest[i] & 15];
        }
        md5cdigest[32] = 0;
        sprintf(buffer,"APOP %s %s\r\n",account->name,md5cdigest);
        pop3_send(sock,buffer);
    }
    else
    {
        sprintf(buffer,"USER %s\r\n",account->name); 
        pop3_send(sock,buffer);
        pop3_recv(sock,buffer);
        sprintf(buffer,"PASS %s\r\n",account->passwd); 
        pop3_send(sock,buffer);
    }
    pop3_recv(sock,buffer);
    if (buffer[0] == '-')
    {
        pop3_disconnect(sock);
        fprintf(stderr,_("%s can't log.\n"),account->name);
	emails->message = (char*) malloc(strlen(buffer)+1);
	strcpy(emails->message, buffer);
        return -2;
    }
    pop3_send(sock,"STAT\r\n");
    if (!pop3_recv(sock,buffer))
    {
        pop3_disconnect(sock);
        return -1;
    }
    if (buffer[0]!='+')
    {
        pop3_disconnect(sock);
        fprintf(stderr,_("Can't stat server\n"));
        return -1;
    }
    sscanf(buffer, "+%*s %d %d", &nb, &size);
    pop3_send(sock,"LAST\r\n");
    if (!pop3_recv(sock,buffer))
    {
        pop3_disconnect(sock);
        return -1;
    }
    if (buffer[0]!='+')
    {
        fprintf(stderr,_("Can't LAST server:unsupported feature ?\n"));
        last=0;
    }
    sscanf(buffer, "+%*s %d", &last);
    VERBOSE(printf(_("%d %s (%d read-%d bytes)\n"), nb, \
		   nb > 1 ? _("messages") : _("message"), last, size));
    pop3_send(sock,"QUIT\r\n");
    pop3_recv(sock,buffer);
    pop3_disconnect(sock);
    emails->total = nb;
    emails->unread = nb-last; 
    return 0;
}
