/*
  WMpop
  Copyright  1999-2002 Jrme SAUTRET (jerome@Sautret.org)

  $Id: wmpop.c,v 1.23 2002/03/11 21:13:47 jerome Exp $

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <X11/Xlib.h>
#include <locale.h>
#include "wmpop.h"
#include "x.h"
#include "pop3.h"
#include "mbox.h"
#include "config_file.h"
#include "sound.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

/* declarations */
extern Display *display;
extern GC      gc;
extern Window  win, icon;
extern Window  root;
extern Pixmap  xpm[2];

struct config configuration;

void main_loop(void);
void arm_conf();
void arm_alarm();
void arm_child();
void check_mail(int);

/* global variables */
int check_mail_interval=10;
int display_emails; /* are we displaying emails ? */
struct pop3_account account;
struct emails_t emails; 
int verbose=0;
Time click_time=0; /* for double clicks */
pid_t pid_conf=0;   /* pid of conf_wmpop */
char file_name[BUF_SIZE];


/* function definitions */

/* run an external command (used when the middle button is pressed)
   returns 0 if the fork is OK
           1 if command is null
          -1 if the fork is KO      
 */
int run_command (char *command)
{
    pid_t pid;
    
    if (command == 0)
    {
	return 1;
    }

    if ((pid = fork ()) < 0)
    {
	perror("run_command, fork");
	return (-1);
    }
    
    if (pid == 0)
    {
	char *argv[4];
	
	argv [0] = "sh";
	argv [1] = "-c";
	argv [2] = command;
	argv [3] = 0;
	execv("/bin/sh", argv);
	perror("run_command, execve");
	exit(127);
    }

    return 0;
}

/* reinit the pid_conf variable when child is dead
 */
void conf_is_dead(int foobar)
{
    int status, code;
    
    DEBUG(printf("Conf is dead\n"));
    switch( waitpid(pid_conf, &status , WNOHANG))
    {
    case -1:
	perror("conf_is_dead");
	return;
    case 0:
	/* it's not conf_wmpop, just the system()
	   from play_sound. Ignore it
	*/
	arm_child();
	return;
    }
    if (WIFEXITED(status))
    {
        code=WEXITSTATUS(status);
        if (code == CONF_QUIT)
            exit(0);
    }
    pid_conf=0;
    if (code != CONF_CLOSE && account.server)
	check_mail(SIGUSR1);
}

/* run the configuration GUI (conf_wmpop)
   arg can take the following values:
   CONF_NORMAL: normal call
*/
void run_config(int arg, char *message)
{
    char *args[10];

    DEBUG(printf("run config\n"))
        if (pid_conf == 0)
	{
	    switch(pid_conf=fork())
            {
	    case -1:
		perror("run_config");
		return;
	    case 0:
		args[0]=CONF_WMPOP;
		args[1]=CONF_PARM;
		switch(arg)
		{
		case CONF_NORMAL:
		    args[2]=file_name;
		    args[3]=NULL;
		    break;
		case CONF_NOFILE:
		    args[2]="nofile";
		    args[3]=file_name;
		    args[4]=NULL;
		    break;
		case CONF_PASS:
		    args[2]="pass";
		    args[3]=file_name;
		    args[4]=message;
		    args[5]=NULL;
		    break;
		case CONF_SERVER:
		    args[2]="server";
		    args[3]=file_name;
		    args[4]=NULL;
		    break;
		}
		execvp(args[0], args); 
		perror("run_config, execvp");
		exit(CONF_QUIT);
	    default:
		DEBUG(printf("pid_conf:%d\n", pid_conf));
		if (message)
		    free(message);
		arm_child();
		break;
            }
	}
}

/*
reinit & configure wmpop using config file
*/
void configure(int signum)
{
    int i;


    DEBUG(printf("begin configure\n"))
    /* reinit */
    for(i=0;i<2;i++)
    {
        if (configuration.pixmap[i])
        {
            XFreePixmap(display, xpm[i]);
        }
    }
    init_default_config(&configuration, &account);
    read_config_file(file_name, &configuration, &account);

    if(configuration.proto != PROTO_MBOX)
    {
	configuration.display_emails = 0;
    }
    
    if (configuration.default_pix)
    {
        VERBOSE(printf(_("Using default pixmaps.\n")))
	    create_default_pixmap(0);
        create_default_pixmap(1);
    }
    else
    {
        VERBOSE(printf(_("Using customs pixmaps.\n")))
	    for(i=0;i<2;i++)
		if (configuration.pixmap[i] && *configuration.pixmap[i]!='\0')
                {
                    if (!create_pixmap(configuration.pixmap[i],i))
                    {
                        fprintf(stderr, \
				_("Can't create pixmap from file %s, "
				  "using default pixmap.\n"),
                                configuration.pixmap[i]);
                        create_default_pixmap(i);
                    }
                }
		else
                {
                    fprintf(stderr, \
			    _("No custom pixmap defined, "
			      "using default pixmap.\n"));
                    create_default_pixmap(i);
                }
    }

    for(i=0;i<2;i++)
    {
        if (configuration.pixmap[i])
        {
            free(configuration.pixmap[i]);
            configuration.pixmap[i]=NULL;
        }
    }
    /* redisplay window (if pixmap have changed after
       reconfiguration) */
    create_cache(NO_CHANGE, 
		 configuration.unread_mail ? emails.unread : emails.total,
		 LED_OFF, display_emails);
    display_app();
    XSync(display, False);

    arm_conf();
    arm_alarm();
}

/* X events management
 */
void main_loop(void)
{
    XEvent event;
    sigset_t set;
    struct timespec delay;

    /* set the refresh delay for the nanosleep */
    delay.tv_sec = 0;                     /* secondes */
    delay.tv_nsec = REFRESH_DELAY * 1000; /* nano-secondes */

    /* create a set to block signals used in wmpop */
    sigemptyset(&set);
    sigaddset(&set, SIGALRM);
    sigaddset(&set, SIGUSR1);
    sigaddset(&set, SIGCHLD);
    
    for (;;)
    {
	/* block signals to prevent a change of pixmap or checking
	   mail during the Xpending: this may cause a XFlush during
	   another XFlush, and hang the app */
	sigprocmask(SIG_BLOCK, &set, NULL);

	while(XPending(display)) 
	{
	    sigprocmask(SIG_UNBLOCK, &set, NULL);
	    XNextEvent(display, &event);

	    /* block signals to prevent a change of 
	       pixmap or checking mail during 
	       redraw of window */
	    sigprocmask(SIG_BLOCK, &set, NULL);


	    switch (event.type)
	    {
	    case Expose:
		DEBUG(printf("Expose\n"));
		if (!event.xexpose.count)
		    display_app();
		break;
	    case ButtonPress:
		if (event.xbutton.button == Button3)
		{
		    DEBUG(printf("Button Press 3\n"));
		    check_mail(SIGUSR1); /* tell check_mail who is running
					    it by using an other signum 
					    than SIGALRM */
		}
		else if (event.xbutton.button == Button2)
		{
		    DEBUG(printf("Button Press 2\n"));
		    VERBOSE(printf(_("Running \"%s\"\n"), 
				   configuration.command));
		    run_command(configuration.command);
		}
		break;
	    case ButtonRelease:
		if (event.xbutton.button == Button1)
		{
		    DEBUG(printf("Button release 1\n"));
		    if ( (event.xbutton.time-click_time) <= 
			 DOUBLE_CLICK_INTERVAL )
		    {
			DEBUG(printf("\t\t\tDouble click\n"));
			click_time=0;
			run_config(CONF_NORMAL, NULL);
		    }
		    else
		    {
			click_time=event.xbutton.time;
		    }
		}
		break;
	    }
	    /* deblock signals */
	}
	sigprocmask(SIG_UNBLOCK, &set, NULL);
	/*usleep(REFRESH_DELAY);*/
	nanosleep(&delay, NULL);
	if (display_emails)
	{
	    sigprocmask(SIG_BLOCK, &set, NULL);
	    update_emails_display(0);
	    display_app();
	    sigprocmask(SIG_UNBLOCK, &set, NULL);
	}
    }
}

/* bind the child death to function conf_is_dead
 */
void arm_child()
{
    struct sigaction sa;
    sigset_t set;

    sa.sa_handler=conf_is_dead;
    /* block other signals in conf_is_dead */
    sigemptyset(&set);
    sigaddset(&set, SIGALRM);
    sigaddset(&set, SIGUSR1);
    sa.sa_mask=set;
    sa.sa_flags=0;
    /* bind conf_is_dead to SIGCHLD */
    if (sigaction(SIGCHLD, &sa, NULL) == -1)
    {
	perror("sigaction in arm_child");
	exit(5);
    }

}

/* bind the configuration function to SIGUSR1
 */
void arm_conf()
{
    struct sigaction sa;
    sigset_t set;

    DEBUG(printf("Begin arm_conf\n"));
    sa.sa_handler=configure;
    /* block SIGALRM (checking mail) during reconfiguration */
    sigemptyset(&set);
    sigaddset(&set, SIGALRM);
    sigaddset(&set, SIGCHLD);
    sa.sa_mask=set;
    sa.sa_flags=0;
    /* bind reconfiguration to SIGUSR1 */
    if (sigaction(SIGUSR1, &sa, NULL) == -1)
    {
	perror("sigaction in arm_conf");
	exit(5);
    }
}

/* bind then checking mail function to SIGALRM
 */
void arm_alarm()
{
    struct sigaction sa;
    sigset_t set;

    sa.sa_handler=check_mail;
    sigemptyset(&set);
    /* block SIGUSR1 (reread of config) when checking mail */
    sigaddset(&set, SIGUSR1);
    sigaddset(&set, SIGCHLD);
    sa.sa_mask=set;
    sa.sa_flags=0;

    if (sigaction(SIGALRM, &sa, NULL) == -1)
    {
	perror("sigaction in arm_alarm");
	exit(5);
    }

    alarm(configuration.check);    
}

/* display a pixmap corresponding to the presence (or not) of a mail
 */
void check_mail(int signum)
{
    int nb_mail, code;
    static int last_nb = -1; /* precedous number of mails */

    /*VERBOSE(printf(_("Checking mail...\n")));*/
    if (signum == SIGALRM)  /* run by alarm signal */
	update_led_display(LED_GREEN);
    else /* run with right button */
    {
	display_emails = 0;
        create_cache(HAVE_NO_MAIL, -1, LED_GREEN, display_emails);
	last_nb = -1;
    }
    display_app();
    /*XSync(display, False);*/

    if (configuration.proto == PROTO_POP3)
	code = pop3_check(&emails, &account);
    else if (configuration.proto == PROTO_MBOX)
	code = mbox_check(&emails);
    else
    {
	fprintf(stderr, _("WMpop internal error: unknown proto %d.\n"), 
		configuration.proto);
	exit(10);
    }

    /* get mail number or unread number depend of configuration */
    if (code == 0)
    { 
	nb_mail=(configuration.unread_mail ? emails.unread : emails.total);
	if (nb_mail != last_nb)
	{
	    if (nb_mail > 0)
	    {
		VERBOSE(printf(_("You have %d %s.\n"), nb_mail, \
			       nb_mail > 1 ? _("mails") : _("mail") ));
		if (configuration.display_emails)
		{
		    display_emails = 1;
		    set_emails_string(emails.froms);
		    update_emails_display(1);
		}
		if (configuration.sound && nb_mail > last_nb) 
		{
		    play_sound(configuration.sound_file);
		}
		if (last_nb < 1)
		{
		    create_cache(HAVE_MAIL, nb_mail, LED_OFF, display_emails);
		}
		else
		{
		    update_number_display(nb_mail);
		    update_led_display(LED_OFF);
		}
	    }
	    else if (nb_mail == 0)
	    {
		VERBOSE(printf(_("You have no mail.\n")));
		display_emails = 0;
		create_cache(HAVE_NO_MAIL, 0, LED_OFF, display_emails);
	    }
	}
	else
	{
	    update_led_display(LED_OFF);
	}
	last_nb = nb_mail;
    }
    else 
    {
	/* account.nb=account.unread=-1; */
	/*create_cache(NO_CHANGE, 0, LED_RED);*/
	update_led_display(LED_RED);

	if (code == -2)
	{
	    run_config(CONF_PASS, emails.message);
	}
	else if (nb_mail == -3)
	{
	    run_config(CONF_SERVER, NULL);
	}
	last_nb = -1;
    }
    display_app();
    /*XSync(display, False);*/
    
    arm_alarm();
}

/* 
the inevitable main function...
*/
int main(int argc, char *argv[])
{
    setlocale (LC_ALL, "");
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);
    x_init(argc,argv);

    config_file_name(file_name, argc, argv);
    while (access(file_name, F_OK) == -1)
    {
        arm_conf();
        run_config(CONF_NOFILE, NULL);
        pause();
    }
	
    configure(SIGUSR2);

    verbose=1;

    check_mail(SIGUSR1);
    main_loop();

    return 0;
}
