/*
  WMpop
  Copyright  1999-2002 Jrme SAUTRET (jerome@Sautret.org)

  $Id: x.c,v 1.22 2002/09/14 08:11:17 jerome Exp $

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; see the file COPYING.  If not, write to
  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>
#include "wmpop.h"
#include "x.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif


#define NUM_WIDTH   6
#define NUM_HEIGTH  9
#define CHAR_WIDTH  6
#define CHAR_HEIGTH 9
#define QUESTION_MARK 10  /* offset of ? in numbers pixmpas */
#define AT_OFFSET  26  /* offset of @ in letters pixmpas */
#define DOT_OFFSET  27  /* offset of . in letters pixmpas */
#define DASH_OFFSET  28  /* offset of - in letters pixmpas */
#define UNDERSCORE_OFFSET  29  /* offset of _ in letters pixmpas */
#define SPACE_OFFSET  30  /* offset of space in letters pixmpas */
#define EMAIL_SIZE 9 /* number of characters used to display e-mails */
#define EMAIL_WIDTH (EMAIL_SIZE * CHAR_WIDTH) 
#define FRAME_WIDTH  14
#define FRAME_HEIGHT 10
#define FRAME_EMAILS_WIDTH  56
#define FRAME_EMAILS_HEIGHT 10

#define XPIX  BORDER_SIZE
#define YPIX  (BORDER_SIZE + BORDER_SIZE/2)
#define XNUMBERS (WIN_SIZE-BORDER_SIZE/2-NUM_WIDTH*2)
#define YNUMBERS (WIN_SIZE-NUM_HEIGTH-(BORDER_SIZE/2))
#define XLED BORDER_SIZE
#define YLED (WIN_SIZE-BORDER_SIZE/2-ledh)
#define XEMAILS (BORDER_SIZE/2)
#define YEMAILS (BORDER_SIZE/4)


/* global variables */
Display *display;
GC      gc, gc_cache, gc_mask;
Window  win, icon;
Window  root;
Pixmap  cache, mask, xpm[2], pixmasks[2], letters, numbers, frame, frame_mask;
Pixmap  frame_emails, frame_emails_mask, led, led_mask;
int ledw, ledh; /* size of the pixmaps */
char *emails_string = NULL; /* string to scroll containing emails */

/* 
initialize X11 stuffs
*/
void x_init(int argc, char *argv[])
{
    unsigned long  white_nb, black_nb;
    int            screen;
    XWMHints       wmhints;
    XSizeHints     sizehints;
    XClassHint     classhint;
    
    display = XOpenDisplay(NULL);
    if (!display)
    {
        fprintf(stderr, _("Can't connect to display.\n"));
        exit(1);
    }
    DEBUG(XSynchronize(display,True))
        screen  = DefaultScreen(display);

    root     = RootWindow(display,screen);
    white_nb = WhitePixel(display,screen);
    black_nb = BlackPixel(display,screen);
    gc       = XDefaultGC(display,screen);

    sizehints.x      = 0;
    sizehints.y      = 0;
    sizehints.width  = WIN_SIZE;
    sizehints.height = WIN_SIZE;
    sizehints.flags  = USSize | USPosition;
    
    win = XCreateSimpleWindow(display, root, sizehints.x, sizehints.y, 
			      sizehints.width, sizehints.height, BORDERWIDTH,
			      black_nb, white_nb);
    XStoreName(display, win, "wmPop3");
    icon = XCreateSimpleWindow(display, root, sizehints.x, sizehints.y, 
                               sizehints.width, sizehints.height, BORDERWIDTH,
                               black_nb, white_nb);
    XSetWMNormalHints(display, win, &sizehints);
    
    wmhints.initial_state = WithdrawnState;
    wmhints.icon_window   = icon;
    wmhints.icon_x        = sizehints.x;
    wmhints.icon_y        = sizehints.y;
    wmhints.window_group  = win;
    wmhints.flags         = StateHint | IconWindowHint | IconPositionHint | 
	WindowGroupHint;
    XSetWMHints(display, win, &wmhints);

    classhint.res_name  = argv[0];
    classhint.res_class = argv[0];
    XSetClassHint(display, win, &classhint);
    
    XSetCommand(display, win, argv, argc);
    
    XSelectInput(display, win,
		 ExposureMask | ButtonPressMask | ButtonReleaseMask );
    XSelectInput(display,
		 icon, ExposureMask | ButtonPressMask | ButtonReleaseMask );

    XMapWindow(display, win);

    init_pixmaps();
}

void init_pixmaps()
{
#   include "letters.xpm"
#   include "numbers.xpm"
#   include "frame.xpm"
#   include "frame_emails.xpm"
#   include "led.xpm"
    XpmAttributes att;

    DEBUG(printf("Screen depth : %d\n", 
		 DisplayPlanes(display, DefaultScreen(display))));

    cache=XCreatePixmap(display, root, WIN_SIZE, WIN_SIZE,
			DisplayPlanes(display, DefaultScreen(display)));
    gc_cache = XCreateGC( display, cache, 0, NULL);
    mask=XCreatePixmap(display, root, WIN_SIZE, WIN_SIZE, 1);
    gc_mask = XCreateGC( display, mask, 0, NULL);

    if (XpmCreatePixmapFromData(display, root, letters_xpm,
				&letters, NULL, NULL) != XpmSuccess)
    {
        printf(_("Error in creating pixmaps for letters\n"));
    }
    if (XpmCreatePixmapFromData(display, root, numbers_xpm,
				&numbers, NULL, NULL) != XpmSuccess)
    {
        printf(_("Error in creating pixmaps for numbers\n"));
    }
    if (XpmCreatePixmapFromData(display, root, frame_emails_xpm,
				&frame_emails, &frame_emails_mask, NULL)
	!= XpmSuccess)
    {
        printf(_("Error in creating pixmaps for emails frame\n"));
    }
    if (XpmCreatePixmapFromData(display, root, frame_xpm,
				&frame, &frame_mask, NULL) != XpmSuccess)
    {
        printf(_("Error in creating pixmaps for numbers frame\n"));
    }
    att.valuemask=0;
    if (XpmCreatePixmapFromData(display, root, led_xpm, &led, &led_mask, &att)
	!= XpmSuccess)
    {
        printf(_("Error in creating pixmaps for led\n"));
    }
    ledw=att.width/3;
    ledh=att.height;
}


int create_pixmap(char *pixmap_file, int n)
{
    int r;

    if(xpm[n])
        XFreePixmap(display, xpm[n]);
    if(pixmasks[n])
        XFreePixmap(display, pixmasks[n]);
    r=XpmReadFileToPixmap(display, root, pixmap_file ,&xpm[n],
			  &pixmasks[n], NULL);
    if(r == XpmSuccess)
	return 1;
    if(r == XpmOpenFailed)
    {
        fprintf(stderr,_("%s: can't find pixmap file\n"),pixmap_file);
        return 0;
    }
    if(r == XpmFileInvalid)
    {
        fprintf(stderr,_("%s: not a valid pixmap file\n"),pixmap_file);
        return 0;
    }
    return 0;
}

int create_default_pixmap(int n)
{
#   include "pix1.xpm"
#   include "pix2.xpm"
    
    char **p;
    
    switch(n)
    {
    case 0:
	p=pix1_xpm;
	break;
    case 1:
	p=pix2_xpm;
	break;
    }

    if(xpm[n])
        XFreePixmap(display, xpm[n]);
    if(pixmasks[n])
        XFreePixmap(display, pixmasks[n]);
    if (XpmCreatePixmapFromData(display, root, p, &xpm[n], &pixmasks[n], NULL)
	== XpmSuccess)
    {
        return 1;
    }
    else
	return 0;
}

/*
return the x position of the number n in numbers pixmap
*/
int number_offset(int n) 
{
    return n*NUM_WIDTH;
}

/*
return the x position of the char c in letters pixmap
*/
int letter_offset(char c) 
{
    switch(c)
    {
    case ' ':
	return SPACE_OFFSET*CHAR_WIDTH;
    case '@':
	return AT_OFFSET*CHAR_WIDTH;
    case '-':
	return DASH_OFFSET*CHAR_WIDTH;
    case '_':
	return UNDERSCORE_OFFSET*CHAR_WIDTH;
    case '.':
	return DOT_OFFSET*CHAR_WIDTH;
    }
    if (c<'a' || c>'z')
	return SPACE_OFFSET*CHAR_WIDTH;
    return (c-'a')*CHAR_WIDTH;
}

void write_letter_pix(char c, int x, int y, GC gc)
{
    if (c >= '0' && c <= '9')
    {
	XCopyArea(display, numbers, cache,  gc, number_offset(c - '0'),
		  0, NUM_WIDTH, NUM_HEIGTH, x, y);
    }
    else
    {
	XCopyArea(display, letters, cache,  gc, letter_offset(c),
		  0, CHAR_WIDTH, CHAR_HEIGTH, x, y);
    }
}

/*
Display the first characters of string s to fill the field.
Add spaces at the end if the string is too short.
*/
void write_string_pix(char *s, int x, int y, GC gc)
{
    int n = 0;

    for (n = 0; n < EMAIL_SIZE ; n++)
    {
	if (*s)
	{
	    write_letter_pix(*s++, x, y, gc);
	}
	else
	{
	    write_letter_pix(' ', x, y, gc);
	}
	x += CHAR_WIDTH;
    } 
}

char* strtolower(char *s)
{
    char *result, *p;
    result = (char*)malloc((strlen(s)+1) * sizeof(char));
    for (p = result ; *s ; s++, p++)
    {
	if (*s >= 'A' && *s <= 'Z')
	{
	    *p = *s + ('a'-'A');
	}
	else
	{
	    *p = *s;
	}
    }
    *p = '\0';
    return result;
}

void set_scrolling_string(char *s)
{
    if (emails_string)
	free(emails_string);
    emails_string = s;
    DEBUG(printf("emails_string:%s\n", emails_string));
    DEBUG(fflush(stdout));
}

void set_emails_string(char *emails[])
{
    char *result, *p;
    int size = EMAIL_SIZE;
    int i;

    for (i=0 ; i < MAX_EMAILS && emails[i] ; i++)
    {
	DEBUG(printf("mail: %s\n", emails[i]));
	size += strlen(emails[i]) + 1;
    }
    result = (char*)malloc(size*sizeof(char));
    for (i=0, p=result ; i < EMAIL_SIZE-1 ; i++, p++)
    {
	*p = ' ';
    }
    *p = '\0';

    for (i=0 ; i < MAX_EMAILS && emails[i] ; i++)
    {
	strcat(result, " ");
	DEBUG(printf("mail: %s\n", emails[i]));
	DEBUG(fflush(stdout));
	strcat(result, emails[i]);
	DEBUG(printf("result(%d/%d):%s\n", strlen(result), size, result));
	DEBUG(fflush(stdout));
    }
    set_scrolling_string(strtolower(result));
    free(result);
}

void update_emails_display(int reinit)
{
    static int pos = 0;
    static char *string = NULL;

    if ( reinit || !string || pos > strlen(string) )
    {
	pos = 0;
	if (string)
	{
	    if (strcmp(string, emails_string) !=0)
	    {
		free(string);
		string = NULL;
	    }    
	}
	if (!string)
	{
	  string = (char *)malloc(sizeof(char)*(strlen(emails_string)+1));
	  strcpy(string, emails_string);
	}

    }
    write_string_pix(string + pos++ , XEMAILS, YEMAILS, gc_cache);
}

void write_number_pix(int n, int x, int y, GC gc)
{
    XCopyArea(display, numbers, cache,  gc, number_offset(n/10),
	      0, NUM_WIDTH, NUM_HEIGTH, x, y);
    XCopyArea(display, numbers, cache,  gc, number_offset(n%10),
	      0, NUM_WIDTH, NUM_HEIGTH, x+NUM_WIDTH, y);
}

/* 
write two question marks at (x,y) coordonates
*/
void write_question_pix(int x, int y, GC gc)
{
    XCopyArea(display, numbers, cache,  gc, number_offset(QUESTION_MARK), 0, 
	      NUM_WIDTH, NUM_HEIGTH, x, y);
    XCopyArea(display, numbers, cache,  gc, number_offset(QUESTION_MARK), 0,
	      NUM_WIDTH, NUM_HEIGTH, x+NUM_WIDTH, y);
}

void update_number_display(int nbmails)
{
    if (nbmails == -1)
	write_question_pix(XNUMBERS, YNUMBERS, gc_cache);
    else
	write_number_pix(nbmails>99 ? 99 : nbmails,
			 XNUMBERS, YNUMBERS, gc_cache);
}

/*
return the x position of the led in pixmap
*/
int led_offset(int color)
{
    return color*ledw;
}

void update_led_display(int color)
{
    DEBUG(printf("Led %d\n", color));
    XCopyArea(display, led, cache, gc_cache,
	      led_offset(color), 0, ledw, ledh, XLED, YLED);
}

void create_cache(int change, int nb_mails, int led_color, int display_emails)
{
    static int pixmap_nb=0;
    static int nbmails=-1;
    
    switch(change)
    {
    case HAVE_MAIL:
	pixmap_nb=1;
	nbmails=nb_mails;
	break;
    case HAVE_NO_MAIL:
	pixmap_nb=0;
	nbmails=nb_mails;	    
	break;
    }

    /* creating the cache */
    /*XFillRectangle(display, cache, gc, 0, 0, WIN_SIZE, WIN_SIZE);*/

    XCopyArea(display, xpm[pixmap_nb], cache,  gc_cache, 0, 0, PIX_SIZE,
	      PIX_SIZE, XPIX, YPIX);

    XCopyArea(display, frame, cache,  gc_cache, 0, 0,
	      FRAME_WIDTH, FRAME_HEIGHT, XNUMBERS, YNUMBERS);
    update_number_display(nb_mails);

    update_led_display(led_color);

    if ( display_emails )
    {
	XCopyArea(display, frame_emails, cache,  gc_cache, 0, 0,
		  FRAME_EMAILS_WIDTH, FRAME_EMAILS_HEIGHT, XEMAILS, YEMAILS);
    }

    /* creating the mask */
    XFillRectangle(display, mask, gc_mask, 0, 0, WIN_SIZE, WIN_SIZE);
    XCopyArea(display, pixmasks[pixmap_nb], mask, gc_mask, 0, 0,
	      PIX_SIZE, PIX_SIZE, XPIX, YPIX);
    if ( display_emails )
    {
	XCopyArea(display, frame_emails_mask, mask,  gc_mask, 0, 0,
		  FRAME_EMAILS_WIDTH, FRAME_EMAILS_HEIGHT, XEMAILS, YEMAILS);
    }
    XCopyArea(display, frame_mask, mask,  gc_mask, 0, 0,
	      FRAME_WIDTH, FRAME_HEIGHT, XNUMBERS, YNUMBERS);
    XCopyArea(display, led_mask, mask,  gc_mask, 0, 0, ledw, ledh,
	      XLED, YLED);
    /*XSetFunction(display, gc_mask, GXinvert);
      XFillRectangle(display, mask, gc_mask, x, y, NUM_WIDTH*2+4, NUM_HEIGTH);
    */

}

/* 
Display the window app
*/
void display_app()
{
    XShapeCombineMask(display, win,  ShapeBounding, 0, 0, mask, ShapeSet);
    XCopyArea(display, cache, win,  gc, 0, 0, WIN_SIZE, WIN_SIZE,
	      0, 0);
    XShapeCombineMask(display, icon,  ShapeBounding, 0, 0, mask, ShapeSet);
    XCopyArea(display, cache, icon, gc, 0, 0, WIN_SIZE, WIN_SIZE,
	      0, 0);
}
